## Dodecahedron
# Open model
mod = Sketchup.active_model 
# All entities in model
ent = mod.entities
ent.clear!
# Pre-define constant sqrt(2) to save calculation time
SQRT2 = Math.sqrt(2.0)
# Define constant for ratio of side to radius
SIDE_TO_RAD_OCTA = SQRT2
# Define constant PHI as the Golden Ratio (used in constructing Dodecahedron and Icosahedron)
PHI = (1.0 + Math.sqrt(5.0))/2 # = 1.618033988749895 approx
# Define constants as the lengths of sides of a Golden Section rectangle which fits inside an Icosahedron
#  with circumsphere radius = 1.0 unit
SHORT_SIDE = 0.52573125081 # Mathematical definitions to replace this in due course
LONG_SIDE = 0.85065072264
# Define constants as the lengths of sides of a Golden Section rectangle which defines 
#   location of points of Dodadecahedron of unit radius r (derived from accurate scale drawing:
#   mathematical derivation to replace this in due course. See diagram for definition of a, b, and c)
DDH_A = 0.607061998207
DDH_B = 0.982246946377
DDH_C = 0.794654472292
# Define constant for ratio of side to radius
SIDE_TO_RAD_DODECA = 0.713644179547
SIDE_TO_RAD_ICOSA = 1.051462501620
# Golden rectangle locating points on Dodecahedron with unit radius has 
#   sides of length DDH_A, DDH_B, DDH_C defined in module initialization above
radius = 1
# Scale size by radius
short_side = radius*DDH_A # of golden rectangle
long_side = radius*DDH_B # of golden rectangle
half_height = radius*DDH_C # of dodecahedron = (DDH_A + DDH_B)/2
delta = half_height - short_side
# Create an empty mesh
numpoly = 12 # faces
numpts = 20 # vertices
mesh = Geom::PolygonMesh.new(numpts, numpoly)
# Define rotations of 36 and 72 degrees about Z_AXIS
rotate_minus36 = Geom::Transformation.rotation ORIGIN, Z_AXIS, -36.degrees
rotate36 = Geom::Transformation.rotation ORIGIN, Z_AXIS, 36.degrees
rotate72 = Geom::Transformation.rotation ORIGIN, Z_AXIS, 72.degrees
# Define four arrays for base, lower shoulder, upper shoulder and top points
# We'll later make the sixth point the same as the first to allow iterations to 'wrap around'
points_base = [6]
points_lower = [6]
points_upper = [6]
points_top = [6]
# Define points as starting vertices of Dodecahedron at each level
points_base[0] = Geom::Point3d.new(short_side, 0, -half_height) # first base level point{
points_lower[0] = Geom::Point3d.new(long_side,0, -delta) # first lower 'shoulder' level point
points_upper[0] = Geom::Point3d.new(long_side,0,delta).transform rotate_minus36 # first upper 'shoulder' level point
points_top[0] = Geom::Point3d.new(short_side, 0, half_height).transform rotate_minus36 # first top point
# First point in each 'row' is already drawn (i=0): 
#  just fill in the gaps and add an extra one on top of the first point
for i in 1..5 
# base points
points_base[i] = Geom::Point3d.new(points_base[i-1]).transform rotate72
# lower 'shoulder' points
points_lower[i] = Geom::Point3d.new(points_lower[i-1]).transform rotate72
# upper 'shoulder' points
points_upper[i] = Geom::Point3d.new(points_upper[i-1]).transform rotate72
# top points
points_top[i] = Geom::Point3d.new(points_top[i-1]).transform rotate72
end
=begin
# Add base points to container (for initial testing)
points_base.each do |point|
if point
  ent.add_cpoint point
end
# p "Point added " + point.inspect
end
points_lower.each do |point|
if point
  ent.add_cpoint point
end
# p "Point added " + point.inspect
end
points_upper.each do |point|
if point
  ent.add_cpoint point
end
# p "Point added " + point.inspect
end
points_top.each do |point|
if point
  ent.add_cpoint point
end
# p "Point added " + point.inspect
end
=end
# Add point at centre to act as pick point for any subsequent Move operation
ent.add_cpoint([0,0,0])
# Create successive faces
# Base - draw clockwise to face outside down
mesh.add_polygon points_base[0], points_base[4], points_base[3], points_base[2], points_base[1]
# First row of surrounding faces
for i in 0..4
 mesh.add_polygon points_base[i], points_base[i+1], points_lower[i+1], points_upper[i+1], points_lower[i]
end #for
# Upper row of surrounding faces
for i in 0..4
 mesh.add_polygon points_lower[i], points_upper[i+1], points_top[i+1], points_top[i], points_upper[i]
end #for
# Top - draw counter-clockwise to face outside up
mesh.add_polygon points_top[0], points_top[1], points_top[2], points_top[3], points_top[4]
# Create faces from the mesh
ent.add_faces_from_mesh mesh, 0 # smooth constant = 0 for no smoothing
# Add centrepoint on bottom face as pick point for potential Move/Copy operations
ent.add_cpoint ([0,0,-half_height])
Sketchup.send_action"viewZoomExtents:"
