## Oniondome
# Open model
mod = Sketchup.active_model 
# All entities in model
ent = mod.entities 
ent.clear!
class Geom::PolygonMesh
# Revolve edges defined by an array of points about an axis
# pts is an Array of points
# axis is an Array with a point and a vector
# numsegments is the number of segments in the rotaion direction
def add_revolved_points(pts, axis, numsegments)
  # Make sure that there are enough points
  numpts = pts.length
  if( numpts < 2 )
      raise ArgumentError, "At least two points required", caller
  end #if
  #TODO: Determine if the points are all in the same plane as the axis
  planar = true
  # Create a transformation that will revolve the points
  angle = Math::PI * 2
  da = angle / numsegments
  t = Geom::Transformation.rotation(axis[0], axis[1], da)
  # Add the points to the mesh
  index_array = []
  for pt in pts do
      if( pt.on_line?(axis) )
          index_array.push( [self.add_point(pt)] )
      else
          indices = []
          for i in 0...numsegments do
              indices.push( self.add_point(pt) )
              #puts "add #{pt} at #{indices.last}"
              pt.transform!(t)
          end
          index_array.push indices
      end
  end
  # Now create polygons using the point indices
  i1 = index_array[0]
  for i in 1...numpts do
      i2 = index_array[i]
      n1 = i1.length
      n2 = i2.length
      nest if( n1 < numsegments && n2 < numsegments )
      for j in 0...numsegments do
          jp1 = (j + 1) % numsegments
          if( n1 < numsegments )
              self.add_polygon i1[0], i2[jp1], i2[j]
              #puts "add_poly #{i1[0]}, #{i2[jp1]}, #{i2[j]}"
          elsif( n2 < numsegments )
              self.add_polygon i1[j], i1[jp1], i2[0]
              #puts "add_poly #{i1[j]}, #{i1[jp1]}, #{i2[0]}"
          else
              if( planar )
                  self.add_polygon i1[j], i1[jp1], i2[jp1], i2[j]
              else
                  # Try adding two triangles instead
                  self.add_polygon i1[j], i1[jp1], i2[jp1]
                  self.add_polygon i1[j], i2[jp1], i2[j]
              end
              #puts "add_poly #{i1[j]}, #{i1[jp1]}, #{i2[jp1]}, #{i2[j]}"
          end
      end
      i1 = i2
  end
end #def
# Extrude points along an axis with a rotation
def add_extruded_points(pts, center, dir, angle, numsegments)
# Make sure that there are enough points
numpts = pts.length
if( numpts < 2 )
    raise ArgumentError, "At least two points required", caller
end
# compute the transformation
vec = Geom::Vector3d.new dir
distance = vec.length
dz = distance / numsegments
da = angle / numsegments
vec.length = dz
t = Geom::Transformation.translation vec
r = Geom::Transformation.rotation center, dir, da
tform = t * r
# Add the points to the mesh
index_array = []
for i in 0...numsegments do
    indices = []
    for pt in pts do
        indices.push( self.add_point(pt) )
        pt.transform!(tform)
    end
    index_array.push indices
end
# Now create polygons using the point indices
i1 = index_array[0]
for i in 1...numsegments do
    i2 = index_array[i]
    
    for j in 0...numpts do
        k = (j+1) % numpts
        self.add_polygon -i1[j], i2[k], -i1[k]
 # Make sure that there are enough points
numpts = pts.length
if( numpts < 2 )
    raise ArgumentError, "At least two points required", caller
end
# compute the transformation
vec = Geom::Vector3d.new dir
distance = vec.length
dz = distance / numsegments
da = angle / numsegments
vec.length = dz
t = Geom::Transformation.translation vec
r = Geom::Transformation.rotation center, dir, da
tform = t * r
# Add the points to the mesh
index_array = []
for i in 0...numsegments do
    indices = []
    for pt in pts do
        indices.push( self.add_point(pt) )
        pt.transform!(tform)
    end
    index_array.push indices
end
# Now create polygons using the point indices
i1 = index_array[0]
for i in 1...numsegments do
    i2 = index_array[i]
    
    for j in 0...numpts do
        k = (j+1) % numpts
        self.add_polygon -i1[j], i2[k], -i1[k]
        self.add_polygon i1[j], -i2[j], -i2[k]
    end
    
    i1 = i2
end
       self.add_polygon i1[j], -i2[j], -i2[k]
    end
    
    i1 = i2
end
end #def
end #Geom::PolygonMesh

module Bezier
def Bezier.eval(pts, t)
degree = pts.length - 1
if degree < 1
    return nil
end
t1 = 1.0 - t
fact = 1.0
n_choose_i = 1
x = pts[0].x * t1
y = pts[0].y * t1
z = pts[0].z * t1
for i in 1...degree
  fact = fact*t
  n_choose_i = n_choose_i*(degree-i+1)/i
  fn = fact * n_choose_i
  x = (x + fn*pts[i].x) * t1
  y = (y + fn*pts[i].y) * t1
  z = (z + fn*pts[i].z) * t1
end
x = x + fact*t*pts[degree].x
y = y + fact*t*pts[degree].y
z = z + fact*t*pts[degree].z
Geom::Point3d.new(x, y, z)
end # method eval
# Evaluate the curve at a number of points and return the points in an array
def Bezier.points(pts, numpts)
curvepts = []
dt = 1.0 / numpts
# evaluate the points on the curve
for i in 0..numpts
  t = i * dt
  curvepts[i] = Bezier.eval(pts, t)
end #for
curvepts
end #def
# Create a Bezier curve in SketchUp
def Bezier.curve(pts, numseg = 16)
entities = mod.active_entities
curvepts = Bezier.points(pts, numseg)
# create the curve
edges = entities.add_curve(curvepts);
edges
end #def
end #Module
#Create the oniondome
mod.start_operation "Oniondome"
r = 2.feet
h = 4.feet
r1 = 3.feet
h1 = 2.feet
r2 = 6.inch
h2 = 3.feet
n1 = 24
n2 = 16
# Create the Bezier curve to sweep
pts = []
pts[0] = [r, 0, 0]
pts[1] = [r1, 0, h1]
pts[2] = [r2, 0, h2]
pts[3] = [0, 0, h]
curvepts = Bezier.points(pts, n2)
# create the mesh and revolve the points
axis = [ORIGIN, Z_AXIS]   
numpts = n1 * n2
numpoly = n1 * (n2-1)
mesh = Geom::PolygonMesh.new numpts, numpoly
mesh.add_revolved_points curvepts, axis, n1
# Create the faces from the mesh
ent.add_faces_from_mesh mesh, 12
Sketchup.send_action "viewZoomExtents:"
mod.commit_operation
    
